//
// JTypist v3.1 - Java typing tutor program for UNIX systems
// Copyright (C) 2000  Simon Baldwin (simonb@sco.com)
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//
//
// KBTranslator class - implements a translation between two keyboard layouts
//

/**
 * Implements a translation between two keyboard layouts.  Note that the
 * character (glyph) must be present in both for the translation to work.
 */
public class KBTranslator {

	// a note of the keyboard layout we are converting from, and the
	// one we are converting to
	private KBLayout	from_kblayout;//	= null;
	private KBLayout	to_kblayout;//		= null;

	
	/**
	 * Create a translator to translate between two given layouts.
	 */
	public KBTranslator( KBLayout from_kblayout, KBLayout to_kblayout ) {
		this.from_kblayout	= from_kblayout;
		this.to_kblayout	= to_kblayout;
	}

	/*
	 * Accessor methods.
	 */
	public KBLayout	get_from_keymap()	{ return ( from_kblayout ); }
	public KBLayout	get_to_keymap()		{ return ( to_kblayout ); }

	/**
	 * Translate a character and shiftstate from one keymap into a char
	 * from the other.
	 */
	public char translate( char character, KeyShiftState shiftstate ) {

		// look up the glyph for this character in the from keymap
		Glyph from_glyph = from_kblayout.find_glyph( character );
		if ( from_glyph != null ) {

			// from this glyph, get the keycode
			KeyCode keycode	= from_glyph.get_keycode();

			// find the keycode of this glyph, and then find the
			// corresponding glyph for that keycode in the to keymap
			Glyph to_glyph = to_kblayout.find_glyph( keycode,
								shiftstate );
			if ( to_glyph != null )

				// translated OK, so return the altered char
				return ( to_glyph.get_character().charValue() );
		}

		// translation failed; the character must be missing from the
		// from map, or the keycode from the to map.  In either case
		// there really isn't much to be done about it - here we play
		// safe and return the original character
		return ( character );
	}


	/**
	 * Printout method.
	 */
	public String toString() {
		String ret_string = getClass().getName() + "{"
				+ "from_kblayout=\"" + from_kblayout
				+ ",to_kblayout=\"" + to_kblayout;
		return ( ret_string + "}" );
	}


	/**
	 * Class test code.
	 */
	public static void main( String args[] ) {
		System.out.println( "*** CLASS TEST CODE ***" );

		KBLayout from = KBLayout.create_kblayout( "US" );
		KBLayout to = KBLayout.create_kblayout( "Dvorak" );
		KBTranslator t = new KBTranslator( from, to );
		System.out.println( "US to Dvorak translation, 32-127" );
		for ( int i = 32; i < 128; i++ ) {
			char c = (char)i;
			System.out.print( "" + c + "="
			+ t.translate( c, KeyShiftState.UNSHIFTED ) + "/"
			+ t.translate( c, KeyShiftState.SHIFTED ) + "/"
			+ t.translate( c, KeyShiftState.META_ALT ) + " " );
			if ( c % 10 == 0 )
				System.out.println();
		}
		System.out.println();
	}
}
